#include "parsetam.h"
#include "XMLTFAppParser.h"
#include "txtfile.h"
                                   
ParseTam::ParseTam(string fileName,Log* outLog) : XMLTFAppParser(fileName,outLog)
{
   ivLog = outLog;
   TiXmlElement* pElem = doc.RootElement();
   LoadVectorSteps(pElem); // attempts to load Teleflow steps into a vector.
}

ParseTam::~ParseTam(void)
{
   
}

// Name:TamParameterType
// Purpose: Retrieves from a tam xml file the parameter type from the parameter field. The
//          parameter type is the attribute value of the'PassedBy=' attribute.
// Note: - 'int parameterValue' specifies which parameter field to obtain the data from.
//       - 'int step' specifies the teleflow step.
//       = 'int *num' is the return value containing either a 1 for passby reference and a
//          0 for passby value.
int ParseTam::TamParameterType(int parameterValue, int step, int* num)
{
   TiXmlNode* pParent = NULL;          //The node pointer that will point to the step node found.
   int parameterType;                  //Will contain the Attribute value from 'PassedBy='

   pParent = GetNode(step);//Retrives the node from the step index.  

   GetParamType(pParent,parameterValue,&parameterType);
   *num = parameterType;

   return 1; 
}

// Name: TamParameterName
// Purpose: Retrieves from a tam xml file the variable or constant between the parameter
//          tags.
// Note: - "string& value' will containt the value found, if no value is found an empty
//          string will be returned. 
int ParseTam::TamParameterName(int parameterValue, int step, string& value)
{
   TiXmlNode* pParent = NULL;          //The node pointer that will point to the step node found.
   string parameterName;               //Will containt the data betweent the parameter tags found. 

   pParent = GetNode(step); 

   GetParamName(pParent,parameterValue,parameterName);
   value = parameterName;

   return 1;
}

// Name: TamParameterDescription
// Purpose: Retrieves the Description attribute value from the parameter tags in a tam
//          xml file.
int ParseTam::TamParameterDescription(int parameterValue, int step, string& value)
{
   TiXmlNode* pParent = NULL;          //The node pointer that will point to the step node found.
   string parameterDescription;        //Will contain the Description attribute value foudn. 

   pParent = GetNode(step); 

   GetParamDescription(pParent,parameterValue,parameterDescription);
   value = parameterDescription;

   return 1; 
}

// Name: TamParameterValue
// Purpose: To check for the existance of a parameter field in a teleflow step. If the
//          'parameterValue' number exists as a valid parameter field in the xml file
//          true is returned otherwise false is returned. 
bool ParseTam::TamParameterValue(int parameterValue,int step)
{
   TiXmlNode* pParent = NULL;          //The node pointer that will point to the step node found.
   string parameterString;

   pParent = GetNode(step); 
   
   GetParamValue(pParent,parameterValue,parameterString);

   if(parameterString == "")
   {
      return 0;
   }
   else
   {
      return 1;
   }
}
//Cleans up the stepRec pointers allocated in FetchXmlStep
void ParseTam::FreeStepRecMemory(PSTEPREC stepRec)
{
   delete [] stepRec->label.arr;
   delete [] stepRec->s1.arr;
   delete [] stepRec->s2.arr;
   delete [] stepRec->s3.arr;
   delete [] stepRec->s4.arr;
   delete [] stepRec->s5.arr;
   delete [] stepRec->s6.arr;
   delete [] stepRec->s7.arr;
   delete [] stepRec->s8.arr;
   delete [] stepRec->s9.arr;
   delete [] stepRec->s10.arr;
}

// Name: FetchXmlStep
// Purpose: FetchXml steps reads all information from a single teleflow xml step and
//          loads it into the stepRec structure. If it fails to read in a field IVR_FAILURE
//          gets returned otherwise IVR_SUCCESS gets returned. 
IV_STATUS ParseTam::FetchXmlStep(int step, PSTEPREC stepRec)
{
   int number;
   int unsigned index = 1; 
   string data;

   int paramType;
   string stringType;
   
   string paramValue;
   string paramName;
   string paramDescription; 

   //Load all the Records from the xml and store them in stepRec.
   if(GetRecordNumeric(step, "Function",&number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->op = number;

   if(GetRecordString(step, "Label", data) == 0)
   {
      return IVR_FAIL;
   }
   ConvertToMyChar(data,stepRec->label);//allocates memory for MYCHAR.arr and copies string into it. 

   if(GetRecordNumeric(step, "Current", &number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->current = number;

   if(GetRecordNumeric(step, "Next", &number) == 0)//is a numeric Record.
   {
      return IVR_FAIL;
   }
   stepRec->next = number;

   if(GetRecordNumeric(step, "Attributes",&number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->attrib = number; 

   if(GetRecordNumeric(step, "Parent", &number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->parent = number; 

   //Get The Numeric Fields n1-n8
   if(GetFieldNumeric(step, 1, &number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->u1 = number;

   if(GetFieldNumeric(step,2,&number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->u2 = number;

   if(GetFieldNumeric(step, 3, &number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->u3 = number;

   if(GetFieldNumeric(step, 4, &number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->u4 = number;

   if(GetFieldNumeric(step, 5, &number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->u5 = number;

   if(GetFieldNumeric(step, 6, &number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->u6 = number;

   if(GetFieldNumeric(step, 7, &number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->u7 = number;

   if(GetFieldNumeric(step, 8, &number) == 0)
   {
      return IVR_FAIL;
   }
   stepRec->u8 = number; 

   //Get the string fields s1 - s10
   if(GetFieldString(step, 1, data) == 0)
   {
      return IVR_FAIL;
   }
   ConvertToMyChar(data,stepRec->s1);

   if(GetFieldString(step, 2, data) == 0)
   {
      return IVR_FAIL;
   }
   ConvertToMyChar(data,stepRec->s2);

   if(GetFieldString(step, 3, data) == 0)
   {
      return IVR_FAIL;
   }
   ConvertToMyChar(data,stepRec->s3);

   if(GetFieldString(step, 4, data) == 0)
   {
      return IVR_FAIL;
   }
   ConvertToMyChar(data,stepRec->s4);

   if(GetFieldString(step, 5, data) == 0)
   {
      return IVR_FAIL;
   }
   ConvertToMyChar(data,stepRec->s5);

   if(GetFieldString(step, 6, data) == 0)
   {
      return IVR_FAIL;
   }
  
   if(data == "")//check for parameters.
   {
      GetRecordNumeric(step,"Function",&number);

      if(number == 30)//proc Start.
      {
         for(index =1; index < 11;index++)//are 10 entries max!
         {
            if(TamParameterValue(index,step) != 0)
            {
                TamParameterDescription(index, step,paramDescription);
                TamParameterName(index, step, paramName);
                TamParameterType(index,step,&paramType);

                ConvertToString(paramType,stringType);
                data += paramName + END_SUBFIELD + stringType + END_SUBFIELD + paramDescription + END_SUBFIELD + END_SUBSECT;
            }
            else
            {
              continue;
            }
         }
      }
      else if(number == 500 || number == 505)//RunFlowChart or RunTap has no GetParamType since it doesnt specify reference or pass by value.
      {
         for(index = 1;index < 11;index++)
         {
            if(TamParameterValue(index, step) != 0)
            {
               TamParameterName(index, step, paramName);
               data += paramName + END_SUBFIELD + END_SUBSECT;
            }
            else
            {
               continue;  //might be something wrong with this....
            }
         }

      }
   }

   ConvertToMyChar(data,stepRec->s6); //Is used to store the parameters from a tam step.

   if(GetFieldString(step, 7, data) == 0)
   {
      return IVR_FAIL;
   }
   ConvertToMyChar(data,stepRec->s7);

   if(GetFieldString(step, 8, data) == 0)
   {
      return IVR_FAIL;
   }
   ConvertToMyChar(data,stepRec->s8);

   if(GetFieldString(step, 9, data) == 0)
   {
      return IVR_FAIL;
   }
   ConvertToMyChar(data,stepRec->s9);

   if(GetFieldString(step, 10, data) == 0)
   {
      return IVR_FAIL;
   }
   ConvertToMyChar(data,stepRec->s10);

   return IVR_SUCCESS;
}

// Name:LoadChildStep
// Purpose: loads all the childsteps right after the teleflow step has been loaded.
//          In the childstep node it adds a new Parent attribute with the attribute value of
//          the parent step number. This value is used by FetchXmlStep.
int ParseTam::LoadChildStep(TiXmlNode* pParent)
{
   //Get all the childeren from the parent node
   TiXmlNode* childnode = 0;           //contains the XML Tag
   TiXmlElement* pElement = 0;         //Used to convert to pElement.
   int value;                          //will hold the 'current=' attribute value of the teleflowstep. 
   int errorCode;                      //The errorcode create by QueryValueAttribute
   
   pElement = pParent->ToElement();//try to convert the parent node to an element

   if(pElement == NULL)
   {
      return 0;
   }

   errorCode = pElement->QueryValueAttribute("Current",&value);//Get the Current Number from the parrent node

   if(errorCode == TIXML_NO_ATTRIBUTE)
   {
      value = 0; //default value for Parent attribute.  
   }
   
   childnode = pParent->IterateChildren(childnode); //Get the  first next node

   if(childnode == NULL)
   {
      return 0;
   }

   while(childnode != NULL)
   {
      if(childnode->ValueStr() == "Child")
      {   
         pElement = childnode->ToElement();        //has to be an element since we found it to be "Child"
         pElement->SetAttribute("Parent", value);  //Add a parent attribute to the xml structure so that we have the proper parrent number in the child node.
         VectorStepNodes.push_back(childnode);     //is the child node
      }
      childnode = pParent->IterateChildren(childnode);
   }

   return 1;
}

// Name: LoadVectorSteps
// Purpose: Loads all the teleflow steps and child steps from the xml tam file into a vector.
int ParseTam::LoadVectorSteps(TiXmlElement* pElem)
{                     
   TiXmlNode* pNxtSib; //a pointer to the next step node
   TiXmlNode* pChild;  //a pointer to the current step node

   if(pElem == NULL) //Failed to load the document
   {
      return 0; 
   }
   
   pChild = pElem->FirstChild();//Get the first Child Tag <Teleflow Step1>
   if(pChild == NULL)
   {
      return 0; 
   }
   do
   {
      if(pChild->ValueStr() == "TeleFlowStep")
      {
          VectorStepNodes.push_back(pChild);
          LoadChildStep(pChild);
      }

      pNxtSib = pChild->NextSibling();//Get the next step
      pChild = pNxtSib;

   }
   while(pNxtSib != NULL);

   return 1; 
}




