#include <stdio.h>             // Needed for file open, read & write.
#include <stdlib.h>             // Needed for "atoi".
#include "step_textspeech.h"
#include "..\BoardDLLs\brd_generic.h"
#ifdef TF_CLOSED_SOURCE
#include "..\tts\tts_code.h"
#endif

StepTextSpeech::StepTextSpeech(STEPREC& stepRecord) : Step(stepRecord)
{
   bInt = (bool) stepRecord.u1;
   text = stepRecord.s6.arr;
   serverIP = stepRecord.s1.arr;
   serverIPPort = stepRecord.s2.arr;
   connectionTimeout = stepRecord.s3.arr;
   timeout = stepRecord.s4.arr;
}

StepTextSpeech::~StepTextSpeech(void)
{
}

const char* StepTextSpeech::GetStepName(void) const
{
	return("Speak Text");
}

IV_STATUS StepTextSpeech::Execute(PIVRINFO pInfo)
{
   IV_STATUS status = IVR_SUCCESS;	// status returned from this module
   string    parsedText;						// parsed version of text to be converted
   string    parsedServerIP;					// parsed version of TTS server IP
	char*     value;			   		// temporary string for conversion purposes
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
	int       serverIPport;				// parsed version of TTS server IP port
   int          iConnectTimeout;    // integer versions of timeouts, after parsing
   int          iTimeout;           //  "
   char*        buffer = NULL;      // buffer to receive converted audio
   unsigned int bufsize = 0;        // size of the audio buffer
   SOUND_FORMAT playbackFormat;     // audio format required by current Board DLL

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

	// Parse server IP field
   if((value = pInfo->variables->ParseParameter("TTS server", serverIP.c_str(), "127.0.0.1")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
	   parsedServerIP = value;
      delete[] value;
   }

	// Parse server IP port field
   if((value = pInfo->variables->ParseParameter("TTS server port", serverIPPort.c_str(), "7000")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      serverIPport = atoi(value);
      delete[] value;

   	if(serverIPport <= 0)
   	{
	   	pInfo->ivLog->Print("      ERROR: Invalid IP port!\n");
         bParmsValid = false;
   	}
   }

	// Parse text
   if((value = pInfo->variables->ParseParameter("Text", text.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedText = value;
      delete[] value;
   }

   // Parse overall timeout
   if((value = pInfo->variables->ParseParameter("Overall timeout", timeout.c_str(), "@TIMEOUT")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iTimeout = atoi(value);
      delete[] value;

      if((iTimeout < 0) && (iTimeout != FOREVER))
      {
         pInfo->ivLog->Print("      ERROR: Timeout '%s' must be >= 0\n",
            pInfo->variables->IsParsedEmpty(timeout.c_str()) ? "@TIMEOUT" : timeout.c_str());
         bParmsValid = false;
      }
   }

   // Parse connection timeout
   if((value = pInfo->variables->ParseParameter("Initial Connection timeout", connectionTimeout.c_str(), "500")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iConnectTimeout = atoi(value);
      delete[] value;

      if((iConnectTimeout <= 0) && (iTimeout != FOREVER))
      {
         pInfo->ivLog->Print("      ERROR: Initial Connection timeout '%s' must be > 0\n",
                             connectionTimeout.c_str());
         bParmsValid = false;
      }
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   if(pInfo->portLicense == UNLICENSED)
   {
      pInfo->ivLog->Print ("      THIS PORT IS UNLICENSED\n");
      return(IVR_NOTSUPPORTED);
   }

   // Get board level audio format (will be different for the different board types)
   playbackFormat = pInfo->ivBoard->GetPreferredFormat();

   // Output TTS conversion parameters to the log
   switch(playbackFormat)
   {
      case TF_BY_EXTENSION: value = "TF_BY_EXTENSION"; break;
      case TF_WAV_8KHZ_16BIT: value = "TF_WAV_8KHZ_16BIT"; break;
      case TF_VCE_24: value = "TF_VCE_24"; break;
      case TF_VOX: value = "TF_VOX"; break;
      case TF_WAV_8KHZ_8BIT_MULAW: value = "TF_WAV_8KHZ_8BIT_MULAW"; break;
      case TF_WAV_8KHZ_16BIT_G610: value = "TF_WAV_8KHZ_16BIT_G610"; break;
      case TF_WAV_8KHZ_8BIT_ALAW: value = "TF_WAV_8KHZ_8BIT_ALAW"; break;
      default: value = "UNKNOWN"; break;
   }
   pInfo->ivLog->Print("      Converting text to audio format %s (%i)\n", value, playbackFormat);

#ifdef TF_CLOSED_SOURCE
   // Submit text to TTS server to convert to audio
   status = TTSConvert(pInfo->ivLog,             // Log* ivLog
                       parsedServerIP.c_str(),   // const char* pszServerIP
                       serverIPport,             // int nPort
                       parsedText.c_str(),       // const char* text
                       parsedText.length(),      // int textlen
                       playbackFormat,           // SOUND_FORMAT tfAudioFormat
                       buffer,                   // char*& pDestBuffer
                       bufsize,                  // unsigned int& bufsize
                       iConnectTimeout,          // unsigned int connectTimeout
                       iTimeout);                // unsigned int overallTimeout
#else
   buffer = NULL;
   status = IVR_NOTSUPPORTED;
#endif

   if(status == IVR_SUCCESS)
   {
      // Submit the resulting buffer to the board to play the converted audio
      if (buffer == NULL)
      {
         pInfo->ivLog->Print("      ERROR converting text!\n");
         status = IVR_FAIL;
      }
      else
      {
         pInfo->ivLog->Print("      Playing audio buffer\n");
         status = pInfo->ivBoard->AudioPlayBuffer(buffer,
                                                  bufsize,
                                                  bInt,
                                                  playbackFormat,
                                                  atoi(pInfo->variables->Get("@VOLUME")));
         delete[] buffer;
      }
   }

   return(status);
}
//StepTextSpeech::Execute

