/* -------------------------------------------------------------------------
   File:        BRD_LOAD.CPP

   Purpose:     These functions are used to map and unmap functions in a
                board DLL into the address of the current process by
                dynamically loading the DLL at runtime.

                Now the EXE can be generic and dynamically linked with ANY
                vendor board DLL.  This eliminates the need to create
                multiple EXEs - one per board vendor.

   Notes:       - This file only needs to be modified if a new function will
                  be exported from ALL board DLLs.
   ------------------------------------------------------------------------- */
#include <stdio.h>
#include "brd_load.h"

HINSTANCE           hlib;
TCreateInstance     BrdCreateInstance;
TReleaseInstance    BrdReleaseInstance;
TOpenHardwareDriver BrdOpenHardwareDriver;
TCloseHardwareDriver BrdCloseHardwareDriver;

static void FunctionNotFound (Log *sysLog, char *fnNm);

/* -------------------------------------------------------------------------
   Purpose:     Loads in a function from the Board DLL.

   Notes:       - Functions must be exported using 'extern "C"' to prevent
                  C++ from performing name mangling.
                - Functions are exported using the C calling sequence (using
                  the __cdecl keyword).  This is the default for Borland.
                - The C calling sequence prefixes function names with
                  underscores and names are case sensitive.
   ------------------------------------------------------------------------- */
#define LOAD_DLL_FN(fn)                                                       \
           Brd##fn = (T##fn) GetProcAddress (hlib, "_" #fn);                  \
           if (!Brd##fn) {                                                    \
              Brd##fn = (T##fn) GetProcAddress (hlib, #fn);                   \
              if (!Brd##fn) {                                                 \
                 FunctionNotFound (sysLog, #fn);                              \
                 return NULL;                                                 \
              }                                                               \
           }

/* -------------------------------------------------------------------------
   Name:        LoadBoardFunctions

   Purpose:     Attempt to load the board functions from the specified board
                DLL.  If not successful, all function pointers will be NULL.
   ------------------------------------------------------------------------- */
HINSTANCE LoadBoardFunctions(Log* sysLog, char* pszBoardDLL)
{
   if(sysLog != NULL)
	{
      sysLog->Print ("LoadBoardFunctions: Start\n");
}
   UnloadBoardFunctions();
   hlib = LoadLibrary (pszBoardDLL);

   if(hlib != NULL)
   {
      LOAD_DLL_FN(CreateInstance)
      LOAD_DLL_FN(ReleaseInstance)
      LOAD_DLL_FN(OpenHardwareDriver)
      LOAD_DLL_FN(CloseHardwareDriver)

      if(sysLog != NULL)
      {
         sysLog->Print("LoadBoardFunctions: Success.\n");
      }
   }

   return(hlib);
}
/*LoadBoardFunctions*/

/* -------------------------------------------------------------------------
   Name:        UnloadBoardFunctions

   Purpose:     Removes (unmaps) the DLL library from the address space of
                the calling process.
   ------------------------------------------------------------------------- */
void UnloadBoardFunctions()
{
   if (hlib) {
      FreeLibrary (hlib);
      hlib = 0;
   }

   BrdCreateInstance    = 0;
   BrdReleaseInstance   = 0;
}
/*UnloadBoardFunctions*/

/* -------------------------------------------------------------------------
   Name:        CreateInstance
   ------------------------------------------------------------------------- */
BoardGeneric* CreateInstance (PIVRINFO pInfo)
{
   BoardGeneric* gen;

   if (BrdCreateInstance) {
      gen = BrdCreateInstance (pInfo);
      return gen;
   }
   return 0;
}
/*CreateInstance*/

/* -------------------------------------------------------------------------
   Name:        ReleaseInstance
   ------------------------------------------------------------------------- */
void ReleaseInstance (BoardGeneric *obj)
{
   if (BrdReleaseInstance)
      BrdReleaseInstance (obj);
}
/*ReleaseInstance*/

/* -------------------------------------------------------------------------
   Name:        OpenHardwareDriver
   ------------------------------------------------------------------------- */
bool OpenHardwareDriver (Log *sysLog)
{
   if (BrdOpenHardwareDriver != NULL)
   {
      return(BrdOpenHardwareDriver(sysLog));
   }
   return(false);
}
//OpenHardwareDriver

/* -------------------------------------------------------------------------
   Name:        CloseHardwareDriver
   ------------------------------------------------------------------------- */
bool CloseHardwareDriver (Log *sysLog)
{
   if (BrdCloseHardwareDriver != NULL)
   {
      return(BrdCloseHardwareDriver(sysLog));
   }
   return(false);
}
//CloseHardwareDriver

/* -------------------------------------------------------------------------
   Name:        FunctionNotFound

   Purpose:     If 'LoadBoardFunctions' is unable to load a function, this
                function is called to print an error message and unload the
                DLL.

   Notes:       - See the notes for the macro 'LOAD_BOARD_FN'.
   ------------------------------------------------------------------------- */
void FunctionNotFound (Log *sysLog, char *fnNm)
{
   if (sysLog) {
      DWORD   errorID = GetLastError();
      char    MsgBuf[1024];

      FormatMessage (
         FORMAT_MESSAGE_FROM_SYSTEM,
         0,
         errorID,
         MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
         MsgBuf,
         1024,
         0
      );
      if (sysLog)
         sysLog->Print ("ERROR: Unable to load '_%s' or '%s':\n\t%x: %s",
                        fnNm, fnNm, errorID, MsgBuf);
   }
}
/*FunctionNotFound*/

