/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
#include <string.h>	// strlen, strcpy, strncpy, strcat, stricmp, strchr
#include "iv_str.h"	// strtrim
#include "ini.h"

#if defined(DEBUG)
#define DMSG(s)  printf s
#else
#define DMSG(s)
#endif

/* -------------------------------------------------------------------------
   Name:        INIFile::INIFile
   ------------------------------------------------------------------------- */
INIFile::INIFile (const char *fileNm)
{
   DMSG (("INIFile::INIFile: File Name '%s'\n", fileNm));
   pszFileNm = 0;
   fINI = 0;
   bInSection = false;
   if (fileNm && *fileNm) {
      pszFileNm = new char [strlen (fileNm) + 1];
      strcpy (pszFileNm, fileNm);
   }
}
/*INIFile::INIFile*/

/* -------------------------------------------------------------------------
   Name:        INIFile::~INIFile
   ------------------------------------------------------------------------- */
INIFile::~INIFile()
{
   DMSG (("INIFile::~INIFile\n"));
   Close();
   delete [] pszFileNm;
}
/*INIFile::~INIFile*/

/* -------------------------------------------------------------------------
   Name:        INIFile::Open


   Purpose:     Opens the file and sets the section flag to false.
   ------------------------------------------------------------------------- */
STATUS INIFile::Open()
{
   DMSG (("INIFile::Open\n"));
   if ((fINI = fopen (pszFileNm, "r")) == NULL)
      return FAILURE;

   bInSection = false;
   return SUCCESS;
}
/*INIFile::Open*/

/* -------------------------------------------------------------------------
   Name:        INIFile::Close

   Purpose:     Closes the file.
   ------------------------------------------------------------------------- */
STATUS INIFile::Close()
{
   DMSG (("INIFile::Close\n"));
   if (fINI)
      fclose (fINI);
   fINI = 0;
   bInSection = false;
   return SUCCESS;
}
/*INIFile::Open*/

/* -------------------------------------------------------------------------
   Name:        INIFile::GetLine

   Purpose:     Reads in a line from the INI file.  The linefeed is stripped
                from the end of the line.

   Notes:       - Line must be atleast INI_MAX_LINE_SIZE.
   ------------------------------------------------------------------------- */
void INIFile::GetLine (char *pszLine)
{
   DMSG (("INIFile::GetLine\n"));
   if (!fINI || !pszLine)
      return;

   *pszLine = 0;

   if (!feof (fINI)) {
      fgets (pszLine, INI_MAX_LINE_SIZE, fINI);
      pszLine[INI_MAX_LINE_SIZE - 1] = 0;
   }

   int  iLen = strlen (pszLine);

   if (!feof (fINI))
      if (iLen > 0 && pszLine[iLen - 1] == '\n')
         pszLine[iLen - 1] = 0;
   DMSG (("   Line: '%s'\n", pszLine));
}
/*INIFile::GetLine*/

/* -------------------------------------------------------------------------
   Name:        INIFile::FindSection

   Purpose:     Attempts to locate the specified section within the file.
   ------------------------------------------------------------------------- */
bool INIFile::FindSection (const char *pszSection)
{
   DMSG (("INIFile::FindSection\n"));
   char szLine[INI_MAX_LINE_SIZE];
   char szMatch[INI_MAX_LINE_SIZE];

   if (!fINI)
      return false;

   fseek (fINI, 0L, SEEK_SET);
   bInSection = false;

   szMatch[0] = '[';
   strncpy (szMatch + 1, pszSection, INI_MAX_LINE_SIZE - 2);
   szMatch[INI_MAX_LINE_SIZE - 1] = 0;
   strcat (szMatch, "]");
   DMSG (("FindSection: pszSection = '%s'\n", pszSection));
   DMSG (("FindSection: szMatch    = '%s'\n", szMatch));

   GetLine (szLine);
   while (!bInSection && !feof (fINI)) {
      if (!stricmp (szLine, szMatch))
         bInSection = true;
      else
         GetLine (szLine);
   }
   return bInSection;
}
/*INIFile::FindSection*/

/* -------------------------------------------------------------------------
   Name:        INIFile::FindFirst

   Purpose:     This function attempts to find the first parameter and value
                within the specified section.

   Notes:       - This returns ??? if no file is open.
                - Comment lines start with semicolons ';'.
                - If the equal sign is missing, the line is still considered
                  to be a parameter - it just has the NULL value.
   ------------------------------------------------------------------------- */
long INIFile::FindFirst (const char *pszSection, char *pszParm, char *pszValue)
{
   DMSG (("INIFile::FindFirst\n"));
   if (!fINI)
      return 0L;

   FindSection (pszSection);
   return FindNext (pszParm, pszValue);
}
/*INIFile::FindFirst*/

/* -------------------------------------------------------------------------
   Name:        INIFile::FindNext

   Purpose:     This function attempts to find the next parameter and value
                within the current section.  The search continues from the
                last read.
   ------------------------------------------------------------------------- */
long INIFile::FindNext (char *pszParm, char *pszValue)
{
   DMSG (("INIFile::FindNext\n"));
   char  szLine[INI_MAX_LINE_SIZE];
   char *pszEqual;
   bool  bFoundParameter = false;

   if (!fINI || !bInSection) {
      DMSG (("FindNext: Not in section.\n"));
      return 0L;
   }

   GetLine (szLine);
   while (bInSection && !feof (fINI) && !bFoundParameter) {
      if (*szLine == '[')
         bInSection = false;

      else if (*szLine != ';' && *szLine != ' ' && *szLine) {
         pszEqual = strchr (szLine, '=');
         if (pszEqual)
            *pszEqual = 0;

         strncpy (pszParm, szLine, INI_MAX_LINE_SIZE);
         pszParm[INI_MAX_LINE_SIZE - 1] = 0;
         strtrim (pszParm);

         *pszValue = 0;
         if (pszEqual)
            strncpy (pszValue, pszEqual + 1, INI_MAX_LINE_SIZE);
         pszValue[INI_MAX_LINE_SIZE - 1] = 0;
         strtrim (pszValue);

         bFoundParameter = true;
      }

      if (!bFoundParameter && bInSection)
         GetLine (szLine);
   }
   DMSG (("Parameter: '%s'\n", pszParm));
   DMSG (("Value:     '%s'\n", pszValue));

   return bFoundParameter ? ftell (fINI) : 0L;
}
/*INIFile::FindNext*/

/* -------------------------------------------------------------------------
   Name:        INIFile::Find

   Purpose:     This function attempts to find the specified parameter
                within the specified section.
   ------------------------------------------------------------------------- */
long INIFile::Find (const char *pszSection, const char *pszParm, char *pszValue)
{
   DMSG (("INIFile::Find\n"));
   char  szParameter[INI_MAX_LINE_SIZE];
   long  pos;

   if (!fINI)
      return 0L;

   pos = FindFirst (pszSection, szParameter, pszValue);
   while (pos) {
      if (!stricmp (szParameter, pszParm))
         return pos;
      pos = FindNext (szParameter, pszValue);
   }
   return 0L;
}
/*INIFile::Find*/

/* -------------------------------------------------------------------------
   Name:        INIFile::GetPosition

   Purpose:     Reports the current file position within the file.

   Notes:       - Why would any program need to know this information?
   ------------------------------------------------------------------------- */
STATUS INIFile::GetPosition (long *offset)
{
   DMSG (("INIFile::GetPosition\n"));
   if (!fINI)
      return FAILURE;

   *offset = ftell (fINI);
   return SUCCESS;
}
/*INIFile::GetPosition*/

/* -------------------------------------------------------------------------
   Name:        INIFile::SetPosition

   Purpose:     Sets the position within the INI file.  This will not set
                the file pointer past the end of the file.

   Notes:       - Why would any program need to modify the position?  The
                  'inSection' flag is cleared since new location may not be
                  within section.
   ------------------------------------------------------------------------- */
STATUS INIFile::SetPosition (long offset)
{
   DMSG (("INIFile::SetPosition\n"));
   long origPos;

   GetPosition (&origPos);
   if (offset <= fseek (fINI, 0, SEEK_END)) {
      fseek (fINI, offset, SEEK_SET);
      bInSection = false;
      return SUCCESS;
   }
   fseek (fINI, origPos, SEEK_SET);
   return FAILURE;
}
/*INIFile::SetPosition*/


