#ifndef SPEECHTRAINER_H
#define SPEECHTRAINER_H

#include <winsock2.h>
#include <windows.h>
#include <string>
using std::string;

#include "..\TFServer\ivr_defs.h"
#include "..\Voice\Voice.h"
class BoardGeneric;

#if defined (__DLL__)
#define STRAINERAPI __declspec(dllexport)
#else
#define STRAINERAPI __declspec(dllimport)
#endif

typedef struct
{
   string TAPName;
   string TAMName;
   int    stepNum;
   string stepName;
} STEPSTATISTIC;

typedef struct
{
   STEPSTATISTIC  stepStat;
   string         script;
   string         file;
   bool           completed;
   float          played_sec;
} PLAYSTATISTIC;

class STRAINERAPI SpeechTrainer
{
   public:

      // Speech Trainer constructor
      SpeechTrainer(int portnum, char *pszAppName, GUILog* pGUILog, char* pszLogDir, bool bLogStatus);

      // Sets ivBoard property to BoardGeneric to enable certain system variables:
      // @CALL_ELAPSE, @CALL_END, @CALL_SEIZED, @CALL_START
      void SetBoard(BoardGeneric* board);

      // Update the current TAM (full) name in the SpeechTrainer statistics
      void UpdateTAMInfo(const char* tamName);

      // Save the parameter string passed to the recognizer during SR Acquire step
      void SetSRAcquireParm(const char* parameter);

      void CaptureSpeechDetectTimes(const char* startdate, float starttime, const char* enddate, float endtime);

      // Used to build the file-name portion of the speech audio file
      // SpeechTrainer then knows the file name for debugging purposes
      char* ComposeFileName(int port, const char* extension);

      // Returns the full path where speech trainer files should be written
      // creates the folder if it does not already exists
      // returns an empty string if create fails
      string ComposeTrainerPath(void);

      // Preserve statistical info about Play steps
      // Parameters are passed in from the step.execute code
      // Some values are retrieved from the audioState class of the board
      // Data is manipulated in the SpeechTrainer private properties
      void CapturePlayStats(const char* pszAppFile,      // TAP name
                            int iStepNum,                // step number
                            const char* pszStepName,     // Step name
                            const char* pszScriptcode,   // Script code
                            const char* pszFileName);    // File/path name to audio file

      // Preserve statistical info about SR steps
      // Parameters are passed in from the step.execute code
      // Data is manipulated in the SpeechTrainer private properties
      void CaptureSRStartStats(const char* pszAppFile,        // TAP name
                               int iStepNum,                  // step number
                               const char* pszStepName,       // Step name
                               int iNoVoiceTimeout,           // no voice t/o
                               unsigned int iEndVoiceTimeout, // end speech t/o
                               unsigned int iThresholdConf,   // threshold confidence
                               bool bRetainAudio,             // retainAudio
                               Voice::SR_GRAMMAR srgType,     // grammar type
                               const char* pszBaseGrammar,    // base grammar name/file
                               const char* pszAddGrammar);    // additional grammar text

      // Remember the concept returned by recognizer
      // Some steps translate further (eg: digits from "ONE" to "1)
      // Sets SRHeard property to the concept value passed in
      // Also sets SRAdjusted to same as SRHeard
      // Subsequent call to CaptureSRResult will update SRAdjusted value
      void SetSRConceptHeard(const char* pszConcept);

      // Capture SR result for SpeechTrainer
      void CaptureSRResult(const char cDig,               // touch tone digit
                           const char* pszConcept,        // result of recognition
                           unsigned int iConfidence,      // confidence
                           IV_STATUS status,              // result
                           unsigned int* pHandle);        // to receive SR Handle

      // Flag a previous SR result for review in SpeechTrainer GUI application
      bool FlagSRResult(unsigned int iHandle);            // SR result handle

   private:

      // information stored for internal tracking
      bool            logSRAudio;             // SpeechTrainer needs to know if SR logging on/off
      BoardGeneric*   ivBoard;                // pointer to the engine board
      int             iPort;                  // port number
      GUILog*         ivLog;                  // pointer to the engine log
      string          AppName;                // application name, as identified in TAL file
      string          LogDir;                 // current TeleFlow log directory
      string          currentTAMName;         // full path+name of current TAM
      unsigned int    SRHandle;               // handle assigned to current recognition

      // statistical info that will get written to SpeechTrainer files
      string          recognizerParameter;    // parameter string passed to recognizer upon SR Acquire
      PLAYSTATISTIC   lastPlayedStep;
      PLAYSTATISTIC   lastExecPlayStep;
      STEPSTATISTIC   lastSRStep;
      unsigned int    novoiceTimeout;
      unsigned int    endvoiceTimeout;
      unsigned int    thresholdConf;
      bool            retainAudio;
      Voice::SR_GRAMMAR grammarType;
      string          grammarBase;
      string          grammarBaseData;
      string          grammarAdd;

      string          speechStartDate;
      float           speechStartTime;
      string          speechEndDate;
      float           speechEndTime;

      string          SRAudioFile;
      string          SRHeard;
      string          SRAdjusted;
      unsigned int    SRConfidence;
      IV_STATUS       SRReturn;
      char            termDigit;              // terminating digit, if SR terminated by a key press

      // recognition data stored and marked as "unresolved" for later analysis
      unsigned int    unresolvedSRHandle;    // SR handle
      string          unresolvedStartDate;   // call start date
      string          unresolvedStartTime;   // call start time
      string          unresolvedAudioFile;   // audio file
      //              // port is same as current recognition

      // Private methods
      // ---------------

      // Clears properties to reset SpeechTrainer for new recognition
      // Called by constructor and by WriteSRData after SR stats have been reported to file
      void Invalidate();

      // Write grammar text to a secondary file
      // Returns char* to the file name written
      void WriteGrammar(Voice::SR_GRAMMAR srgType, const char* pszGrammar, int iGrammarCount);

      // Write to trainer file
      void WriteSRData();
};

#undef STRAINERAPI

#endif
