#include <winsock2.h>

// Warnings disabled to avoid warnings generated by Microsoft code
#pragma warn -8027
#pragma warn -8060
#pragma warn -8061
#include <ws2tcpip.h>
// Warnings re-enabled
#pragma warn +8027
#pragma warn +8060
#pragma warn +8061

#include "ace\Time_Value.h"
#include "ace\OS_NS_sys_time.h"
#include "Socket.h"
#include "..\escutil\iv_str.h"
#include "SDPInfo.h"

SDPInfo::SDPInfo(void)
{
   Reset();
}

void SDPInfo::AddHostPayload(SDPInfo::PAYLOAD payload)
{
   AddPayload(hostPayloads, payload);
}

void SDPInfo::AddPayload(deque<SDPInfo::PAYLOAD>& payloadList, SDPInfo::PAYLOAD payload)
{
   deque<SDPInfo::PAYLOAD>::iterator pos;

   // Search for payload in payloads
   for(pos = payloadList.begin(); (pos != payloadList.end()) && (*pos != payload); pos++);

   // Add payload only if it isn't already in the list
   if(pos == payloadList.end())
   {
      payloadList.push_back(payload);
   }
}

void SDPInfo::AddPeerPayload(SDPInfo::PAYLOAD payload)
{
   AddPayload(peerPayloads, payload);
}

void SDPInfo::ClearHostPayloads(void)
{
   hostPayloads.clear();
}

void SDPInfo::ClearPeerPayloads(void)
{
   peerPayloads.clear();
}

const char* SDPInfo::GenerateSessionID(void)
{
   ACE_Time_Value timeOfDay = ACE_OS::gettimeofday();
   char number[14];

   sprintf(number, "%d", timeOfDay.sec());
   sessionID = number;
   sprintf(number, "%d", timeOfDay.usec());
   sessionID += (string) number;

   return(sessionID.c_str());
}

const char* SDPInfo::GetHostAddress(void) const
{
   return(hostAddress.c_str());
}

SDPInfo::PAYLOAD SDPInfo::GetHostPayload(unsigned int index) const
{
   return(index >= hostPayloads.size() ? INVALID_PAYLOAD : hostPayloads[index]);
}

unsigned short SDPInfo::GetHostPort(void) const
{
   return(hostPort);
}

const char* SDPInfo::GetHostUserName(void) const
{
   return(hostUserName.c_str());
}

SDPInfo::MEDIA SDPInfo::GetMediaType(void) const
{
   return(mediaType);
}

const char* SDPInfo::GetPeerAddress(void) const
{
   return(peerAddress.c_str());
}

unsigned short SDPInfo::GetPeerPort(void) const
{
   return(peerPort);
}

const char* SDPInfo::GetPeerUserName(void) const
{
   return(peerUserName.c_str());
}

SDPInfo::PAYLOAD SDPInfo::GetPrimaryPayload(void) const
{
   const deque<SDPInfo::PAYLOAD>* primaryPayloads = prioritiseHostPayloads ? &hostPayloads : &peerPayloads;
   const deque<SDPInfo::PAYLOAD>* secondaryPayloads = prioritiseHostPayloads ? &peerPayloads : &hostPayloads;
   deque<SDPInfo::PAYLOAD>::const_iterator primaryPayload;
   deque<SDPInfo::PAYLOAD>::const_iterator secondaryPayload;

   // Find first primary payload match in secondary payloads
   for(primaryPayload = primaryPayloads->begin(); primaryPayload != primaryPayloads->end(); primaryPayload++)
   {
      for(secondaryPayload = secondaryPayloads->begin(); secondaryPayload != secondaryPayloads->end(); secondaryPayload++)
      {
         if(*secondaryPayload == *primaryPayload)
         {
            return(*primaryPayload);
         }
      }
   }

   return(INVALID_PAYLOAD);
}

SDPInfo::PROTOCOL SDPInfo::GetProtocol(void) const
{
   return(protocol);
}

const char* SDPInfo::GetSessionID(void) const
{
   return(sessionID.c_str());
}

const char* SDPInfo::GetSessionName(void) const
{
   return(sessionName.c_str());
}

const char* SDPInfo::GetVersion(void) const
{
   return(version.c_str());
}

void SDPInfo::Print(Log* ivLog) const
{
   if(ivLog == NULL)
   {
      return;
   }

   ivLog->Print("      SDP\n");
   ivLog->Print("         Session Name: '%s'\n", sessionName.c_str());
   ivLog->Print("         Session ID: '%s'\n", sessionID.c_str());
   ivLog->Print("         Media Type: %d\n", mediaType);
   ivLog->Print("         Protocol: %d\n", protocol);
   ivLog->Print("         Version: '%s'\n", version.c_str());
   ivLog->Print("         Host User Name: '%s'\n", hostUserName.c_str());
   ivLog->Print("         Host Address: '%s'\n", hostAddress.c_str());
   ivLog->Print("         Host Port: %d\n", hostPort);
   ivLog->Print("         Peer User Name: '%s'\n", peerUserName.c_str());
   ivLog->Print("         Peer Address: '%s'\n", peerAddress.c_str());
   ivLog->Print("         Peer Port: %d\n", peerPort);
   // not printed:        prioritiseHostPayloads (bool)
   ivLog->Print("         Host Payloads: ");
   for(unsigned int index = 0; index < hostPayloads.size(); index++)
   {
      ivLog->Print("%d ", hostPayloads[index]);
   }
   ivLog->Print("\n");
   ivLog->Print("         Peer Payloads: ");
   for(unsigned int index = 0; index < peerPayloads.size(); index++)
   {
      ivLog->Print("%d ", peerPayloads[index]);
   }
   ivLog->Print("\n");
   ivLog->Print("         Primary Payload: %d\n", GetPrimaryPayload());
}

void SDPInfo::PrioritiseHostPayloads(void)
{
   prioritiseHostPayloads = true;
}

void SDPInfo::PrioritisePeerPayloads(void)
{
   prioritiseHostPayloads = false;
}

void SDPInfo::Reset(void)
{
   sessionName.clear();
   sessionID.clear();
   hostUserName.clear();
   peerUserName.clear();
   hostAddress.clear();
   peerAddress.clear();
   mediaType = INVALID_MEDIA;
   hostPort = Socket::INVALID_PORT;
   peerPort = Socket::INVALID_PORT;
   protocol = INVALID_PROTOCOL;
   version.clear();
   PrioritisePeerPayloads();
   hostPayloads.clear();
   peerPayloads.clear();
}

void SDPInfo::SetHostAddress(const char* address)
{
   hostAddress = address;
}

void SDPInfo::SetHostPayloads(SDPInfo::PAYLOAD* payloads, int count)
{
   SetPayloads(hostPayloads, payloads, count);
}

void SDPInfo::SetHostPort(unsigned short port)
{
   hostPort = port;
}

void SDPInfo::SetHostUserName(const char* name)
{
   string::size_type userNameStart;
   string::size_type userNameEnd;

   if(name == NULL)
   {
      hostUserName = "";
      return;
   }
   else
   {
      hostUserName = name;
   }

   if((userNameStart = FindCaseInsensitiveString(name, "sip:")) != string::npos)
   {
      userNameStart += 4;
   }
   else if((userNameStart = FindCaseInsensitiveString(name, "sips:")) != string::npos)
   {
      userNameStart += 5;
   }
   else
   {
      userNameStart = 0;
   }

   if(((userNameEnd = FindCaseInsensitiveString(name, "@")) != string::npos) && (userNameEnd >= userNameStart))
   {
      hostUserName = hostUserName.substr(userNameStart, userNameEnd - userNameStart);
   }
   else
   {
      hostUserName = hostUserName.substr(userNameStart);
   }
}

void SDPInfo::SetMediaType(SDPInfo::MEDIA mediaType)
{
   this->mediaType = mediaType;
}

void SDPInfo::SetPayloads(deque<SDPInfo::PAYLOAD>& payloadList, SDPInfo::PAYLOAD* payloads, int count)
{
   payloadList.clear();

   if(payloads == NULL)
   {
      return;
   }

   for(int index = 0; index < count; index++)
   {
      AddPayload(payloadList, payloads[index]);
   }
}

void SDPInfo::SetPeerAddress(const char* address)
{
   peerAddress = address;
}

void SDPInfo::SetPeerPayloads(SDPInfo::PAYLOAD* payloads, int count)
{
   SetPayloads(peerPayloads, payloads, count);
}

void SDPInfo::SetPeerPort(unsigned short port)
{
   peerPort = port;
}

void SDPInfo::SetPeerUserName(const char* name)
{
   peerUserName = name;
}

void SDPInfo::SetProtocol(SDPInfo::PROTOCOL protocol)
{
   this->protocol = protocol;
}

void SDPInfo::SetSessionID(const char* ID)
{
   sessionID = ID;
}

void SDPInfo::SetSessionName(const char* name)
{
   sessionName = name;
}

void SDPInfo::SetVersion(const char* version)
{
   this->version = version;
}
